<?php
namespace HellixElementor\Modules\Accordion\Skins;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use Elementor;
use Elementor\Skin_Base;
use Elementor\Icons_Manager;
use Elementor\Controls_Manager;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use Elementor\Group_Control_Typography;
use HellixElementor\Plugin;
use Elementor\Repeater;
use HellixElementor\Core\Utils as HXUtils;

/**
 * The Card Skin for Accordion Widget class.
 */
class Skin_Accordion_Basic extends Skin_Base {

	/**
	 * Setup the skin
	 *
	 * @param Elementor\Widget_Base $parent The accordion widget.
	 */
	public function __construct( Elementor\Widget_Base $parent ) {
		parent::__construct( $parent );
		add_filter( 'elementor/widget/print_template', array( $this, 'skin_print_template' ), 10, 2 );
		add_action( 'elementor/element/accordion/section_title/before_section_end', [ $this, 'modify_controls' ], 10 );
		add_action( 'elementor/element/accordion/section_toggle_style_content/after_section_end', [ $this, 'modifying_style_sections' ], 10 );
	}

	/**
	 * Get the ID for the skin.
	 */
	public function get_id() {
		return 'hx-accordion-basic';
	}

	/**
	 * Get the title for the skin.
	 *
	 * @return string
	 */
	public function get_title() {
		return esc_html__( 'Basic', 'hellix-elementor' );
	}

	/**
	 * Remove the controls not required for the skin
	 */
	public function modify_controls() {

		$disable_controls = [
			'selected_icon',
			'selected_active_icon',
			'faq_schema',
		];

		foreach ( $disable_controls as $control ) {
			$this->parent->update_control(
				$control,
				[
					'condition' => [
						'_skin!' => 'hx-accordion-basic',
					],
				]
			);
		}

		$this->add_control(
			'light_version',
			[
				'label'     => esc_html__( 'Light Version', 'hellix-elementor' ),
				'type'      => Controls_Manager::SWITCHER,
				'label_on'  => esc_html__( 'Show', 'hellix-elementor' ),
				'label_off' => esc_html__( 'Hide', 'hellix-elementor' ),
				'default'   => '',
			],
			[
				'position' => [
					'at' => 'before',
					'of' => 'title_html_tag',
				],
			]
		);
	}

	/**
	 * Override style section
	 */
	public function modifying_style_sections() {

		$disable_controls = [
			'section_title_style',
			'section_toggle_style_icon',
			'title_padding',
			'content_padding',
			'tab_active_color',
		];

		foreach ( $disable_controls as $control ) {
			$this->parent->update_control(
				$control,
				[
					'condition' => [
						'_skin!' => 'hx-accordion-basic',
					],
				]
			);
		}

		$this->parent->start_injection(
			[
				'of' => 'title_background',
				'at' => 'before',
			]
		);

		$this->parent->add_control(
			'title_class',
			[
				'label'       => esc_html__( 'Title Class', 'hellix-elementor' ),
				'type'        => Controls_Manager::TEXT,
				'default'     => 'text-lg md:text-base fw-600',
				'label_block' => true,
				'description' => esc_html__( 'Additional CSS class that you want to apply to the title', 'hellix-elementor' ),
			]
		);

		$this->parent->end_injection();

		$this->parent->update_control(
			'title_background',
			[
				'selectors' => [
					'{{WRAPPER}} .accordion__button' => 'background-color: {{VALUE}} !important;',
				],
			]
		);

		$this->parent->update_control(
			'title_color',
			[
				'selectors' => [
					'{{WRAPPER}} .hellix-tab-title' => 'color: {{VALUE}} !important;',
				],
			]
		);

		$this->parent->update_control(
			'tab_active_color',
			[
				'selectors' => [
					'{{WRAPPER}} .hellix-tab-title' => 'color: {{VALUE}} !important;',
				],
			]
		);

		$this->parent->update_control(
			'content_background_color',
			[
				'selectors' => [
					'{{WRAPPER}} .accordion__content' => 'background-color: {{VALUE}};',
				],
			]
		);

		$this->parent->start_injection(
			[
				'of' => 'content_background_color',
				'at' => 'before',
			]
		);

		$this->parent->add_control(
			'content_class',
			[
				'label'       => esc_html__( 'Content Class', 'hellix-elementor' ),
				'type'        => Controls_Manager::TEXT,
				'default'     => 'text-sm',
				'label_block' => true,
				'description' => esc_html__( 'Additional CSS class that you want to apply to the <p> tag', 'hellix-elementor' ),
			]
		);

		$this->parent->end_injection();

		$this->parent->update_control(
			'content_color',
			[
				'selectors' => [
					'{{WRAPPER}} .hellix-accordion__content' => 'color: {{VALUE}} !important;',
				],
			]
		);
	}

	/**
	 * Render the widget in the frontend.
	 */
	public function render() {
		$widget        = $this->parent;
		$settings      = $widget->get_settings_for_display();
		$light_version = $settings[ $this->get_control_id( 'light_version' ) ];
		$title         = $settings['title_class'];
		$accordion_id  = uniqid( 'accordion' );

		$widget->add_render_attribute( 'accordion', 'class', array( 'accordion', '-simple', 'js-accordion' ) );
		$widget->add_render_attribute( 'accordion', 'id', $accordion_id );

		if ( 'yes' === $light_version ) {
			$widget->add_render_attribute( 'accordion', 'class', array( '-light', 'md:mt-32' ) );
		}

		?>
		<div <?php $widget->print_render_attribute_string( 'accordion' ); ?>>
		<?php

		foreach ( $settings['tabs'] as $index => $item ) :

			$this->parent->add_render_attribute(
				'title' . $index,
				'class',
				[
					'hellix-tab-title',
					'elementor-accordion-title',
					$settings['title_class'],
					'yes' === $light_version ? ' text-white' : 'text-black',
				]
			);

			$this->parent->add_render_attribute(
				'content' . $index,
				'class',
				[
					$settings['content_class'],
					'yes' === $light_version ? ' text-light' : '',
				]
			);

			$this->parent->add_render_attribute( 'accordion_button' . $index, 'class', 'accordion__button' );
			if ( 'yes' === $light_version ) {
				$this->parent->add_render_attribute( 'accordion_button' . $index, 'class', 'text-white' );
			} else {
				$this->parent->add_render_attribute( 'accordion_button' . $index, 'class', 'text-black' );
			}

			?>
			<div class="accordion__item">
				<div <?php $this->parent->print_render_attribute_string( 'accordion_button' . $index ); ?>>
					<button <?php $this->parent->print_render_attribute_string( 'title' . $index ); ?>><?php echo esc_html( $item['tab_title'] ); ?></button>
					<div class="accordion__icon">
						<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-plus icon"><line x1="12" y1="5" x2="12" y2="19"></line><line x1="5" y1="12" x2="19" y2="12"></line></svg>
						<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-minus icon"><line x1="5" y1="12" x2="19" y2="12"></line></svg>
					</div>
				</div>
				<div class="accordion__content">
					<div class="accordion__content__inner hellix-accordion__content elementor-tab-content">
						<p <?php $this->parent->print_render_attribute_string( 'content' . $index ); ?>>
							<?php echo wp_kses_post( HXUtils::parse_text_editor( $item['tab_content'], $settings ) ); ?>
						</p>
					</div>
				</div>
			</div>
			<?php endforeach; ?>
		</div>
		<?php
		$this->render_script( $accordion_id );
	}

	/**
	 * Render the skin in the editor.
	 *
	 * @param string                $content The content of the skin.
	 * @param Elementor\Widget_Base $widget The accordion widget.
	 * @return string
	 */
	public function skin_print_template( $content, $widget ) {
		if ( 'accordion' === $widget->get_name() ) {
			return '';
		}
		return $content;
	}

	/**
	 * Render script in the editor.
	 *
	 * @param string $accordion_id ID of the accordion.
	 */
	public function render_script( $accordion_id ) {
		if ( Elementor\Plugin::$instance->editor->is_edit_mode() ) :
			?>
			<script type="text/javascript">
				
				 /**
				 * Editor Slider Load.
				 */
				var Accordion = (function() {
					function init() {
						const accordion = document.getElementById( "<?php echo esc_attr( $accordion_id ); ?>" );

						if (!accordion) return;

						const items = accordion.querySelectorAll('.accordion__item');

						if ( ! items ) return;

						for (let l = 0; l < items.length; l++) {
							items[l].addEventListener("click", toggleAccordionItem );
						}
					}

					const toggleAccordionItem = (e) => {
						e.currentTarget.classList.toggle('is-active');
						const content = e.currentTarget.querySelector('.accordion__content');
				
						if (content.style.maxHeight) {
							content.style.maxHeight = null;
						} else {
							content.style.maxHeight = content.scrollHeight + "px";
						}
					};

					return {
						init: init,
					}
				})();
				Accordion.init();
			</script>
			<?php
		endif;
	}
}
