<?php

namespace HellixElementor\Modules\Button\Skins;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use Elementor;
use Elementor\Skin_Base;
use Elementor\Widget_Base;
use Elementor\Icons_Manager;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;

/**
 * Skin for Button.
 */
class Skin_Button_Hellix extends Skin_Base {

	/**
	 * Setup the skin
	 *
	 * @param Elementor\Widget_Base $parent The button widget.
	 */
	public function __construct( Widget_Base $parent ) {
		parent::__construct( $parent );
	}

	/**
	 * Get the ID for the skin.
	 */
	public function get_id() {
		return 'button-hellix';
	}

	/**
	 * Get the title for the skin.
	 *
	 * @return string
	 */
	public function get_title() {
		return esc_html__( 'Hellix', 'hellix-elementor' );
	}

	/**
	 * Register Control Actions Here.
	 */
	protected function _register_controls_actions() {
		add_filter( 'hellix-elementor/widget/button/print_template', [ $this, 'skin_print_template' ], 10, 2 );
		add_action( 'elementor/element/button/section_button/before_section_end', [ $this, 'register_button_content_controls' ], 10 );
		add_action( 'elementor/element/button/section_style/after_section_end', [ $this, 'register_button_style_controls' ], 10 );
	}

	/**
	 * Register button content controls.
	 *
	 * @param array $widget The widget settings.
	 */
	public function register_button_content_controls( $widget ) {

		$disable_controls = [
			'button_type',
			'size',
			'align',
			'',
			'button_css_id',
			'view',
			'icon_align',
			'icon_indent',
		];

		foreach ( $disable_controls as $control ) {
			$widget->update_control(
				$control,
				[
					'condition' => [
						'_skin!' => 'button-hellix',
					],
				]
			);
		}

		$this->add_control(
			'button_type',
			[
				'label'   => esc_html__( 'Type', 'hellix-elementor' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'dark',
				'options' => [
					'light'     => esc_html__( 'Light', 'hellix-elementor' ),
					'simple'    => esc_html__( 'Simple', 'hellix-elementor' ),
					'white'     => esc_html__( 'White', 'hellix-elementor' ),
					'dark'      => esc_html__( 'Dark', 'hellix-elementor' ),
					'secondary' => esc_html__( 'Secondary', 'hellix-elementor' ),
					'accent'    => esc_html__( 'Accent', 'hellix-elementor' ),
					'grey'      => esc_html__( 'Grey', 'hellix-elementor' ),
				],
			],
			[
				'position' => [
					'at' => 'after',
					'of' => 'text',
				],
			]
		);

		$this->add_control(
			'hx-icon_indent',
			[
				'label'     => esc_html__( 'Icon Spacing', 'hellix-elementor' ),
				'type'      => Controls_Manager::SLIDER,
				'range'     => [
					'px' => [
						'max' => 50,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .hx-elementor-button .elementor-align-icon-right' => 'margin-left: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .hx-elementor-button .elementor-align-icon-left' => 'margin-right: {{SIZE}}{{UNIT}};',
				],
			],
			[
				'position' => [
					'at' => 'after',
					'of' => 'selected_icon',
				],
			]
		);

		$this->add_control(
			'icon_size',
			[
				'label'      => esc_html__( 'Icon Size', 'hellix-elementor' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'selectors'  => [
					'{{WRAPPER}} .hx-button-icon' => 'font-size: {{SIZE}}{{UNIT}} !important;',
				],
			],
			[
				'position' => [
					'at' => 'after',
					'of' => 'icon_indent',
				],
			]
		);
	}

	/**
	 * Register Button Style controls.
	 *
	 * @param array $widget The widget settings.
	 * @return void
	 */
	public function register_button_style_controls( $widget ) {

		$this->parent     = $widget;
		$disable_controls = [
			'section_style',
		];

		$widget->update_control(
			'section_style',
			[
				'condition' => [
					'_skin!' => 'button-hellix',
				],
			]
		);

		$this->start_controls_section(
			'section_style',
			[
				'label' => esc_html__( 'Button', 'hellix-elementor' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'typography',
				'global'   => [
					'default' => Global_Typography::TYPOGRAPHY_ACCENT,
				],
				'selector' => '{{WRAPPER}} .hellix-elementor-button-text',
			]
		);

		$this->add_control(
			'button_text_color',
			[
				'label'     => esc_html__( 'Text Color', 'hellix-elementor' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '#615E5E',
				'selectors' => [
					'{{WRAPPER}} .hellix-elementor-button-text' => 'fill: {{VALUE}}; color: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'button_bg_color',
			[
				'label'     => esc_html__( 'Background Color', 'hellix-elementor' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .hellix-button' => 'background-color:{{VALUE}} !important;',
				],
			]
		);

		$this->add_control(
			'disable_padding',
			[
				'label'        => esc_html__( 'Disable padding', 'hellix-elementor' ),
				'type'         => Controls_Manager::SWITCHER,
				'label_on'     => esc_html__( 'Yes', 'hellix-elementor' ),
				'label_off'    => esc_html__( 'No', 'hellix-elementor' ),
				'return_value' => 'yes',
				'default'      => 'no',
			]
		);

		$this->add_control(
			'btn_css',
			[
				'label'       => esc_html__( 'CSS Classes', 'hellix-elementor' ),
				'type'        => Controls_Manager::TEXT,
				'dynamic'     => [
					'active' => true,
				],
				'title'       => esc_html__( 'Add your custom class WITHOUT the dot. e.g: my-class', 'hellix-elementor' ),
				'description' => esc_html__( 'Added to .button class', 'hellix-elementor' ),
			]
		);

		$this->add_control(
			'icon_wrap_css',
			[
				'label'       => esc_html__( 'CSS Classes', 'hellix-elementor' ),
				'type'        => Controls_Manager::TEXT,
				'dynamic'     => [
					'active' => true,
				],
				'title'       => esc_html__( 'Add your custom class WITHOUT the dot. e.g: my-class', 'hellix-elementor' ),
				'description' => esc_html__( 'Added to <span> tag class', 'hellix-elementor' ),
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Rendet the text
	 */
	public function render_text() {
		$parent        = $this->parent;
		$settings      = $parent->get_settings_for_display();
		$button_type   = $this->get_instance_value( 'button_type' );
		$icon_wrap_css = $this->get_instance_value( 'icon_wrap_css' );
		$parent->add_render_attribute(
			[
				'text'    => [
					'class' => 'hellix-elementor-button-text',
				],
				'wrapper' => [
					'class' => [ 'elementor-button-icon hx-button-icon', 'elementor-align-icon-left', $icon_wrap_css ],
				],
			]
		);

		if ( ! empty( $settings['icon'] ) || ! empty( $settings['selected_icon']['value'] ) ) :
			?>
			<span <?php $parent->print_render_attribute_string( 'wrapper' ); ?>> 
				<?php
				Icons_Manager::render_icon(
					$settings['selected_icon'],
					[
						'aria-hidden' => 'true',
						'class'       => [ 'icon ' ],
					]
				);
				?>
			</span>
			<?php
		endif;

		if ( ! empty( $settings['text'] ) ) {
			?>
			<span <?php $parent->print_render_attribute_string( 'text' ); ?>><?php echo wp_kses_post( $settings['text'] ); ?></span>
			<?php
		}
		?>
		<?php
	}

	/**
	 * Render the widget in the frontend.
	 */
	public function render() {
		$parent   = $this->parent;
		$settings = $parent->get_settings_for_display();

		$skin_control_ids = [
			'button_type',
			'icon_wrap_css',
			'btn_css',
			'disable_padding',
		];

		$skin_settings = [];

		foreach ( $skin_control_ids as $skin_control_id ) {
			$skin_settings[ $skin_control_id ] = $this->get_instance_value( $skin_control_id );
		}

		if ( ! empty( $settings['button_css_id'] ) ) {
			$parent->add_render_attribute( 'button', 'id', $settings['button_css_id'] );
		}

		if ( ! empty( $settings['link']['url'] ) ) {
			$parent->add_link_attributes( 'button', $settings['link'] );
		}

		$button_class = [ 'hellix-button button ' ];

		if ( ( 'simple' !== $skin_settings['button_type'] ) && ! empty( $settings['selected_icon']['value'] ) && 'yes' !== $skin_settings['disable_padding'] ) {
			$button_class[] = '-md';
		}

		$button_class[] = '-' . $skin_settings['button_type'];

		if ( ! empty( $skin_settings['btn_css'] ) ) {
			$button_class[] = $skin_settings['btn_css'];
		}
		$parent->add_render_attribute( 'button', 'class', 'hx-elementor-button' );
		$parent->add_render_attribute( 'button', 'class', $button_class );

		?>
		<a <?php $parent->print_render_attribute_string( 'button' ); ?>>
			<?php $this->render_text(); ?>		
		</a>
		<?php
	}

	/**
	 * Print Template.
	 *
	 * @param string                $content The skin output string.
	 * @param Elementor\Widget_Base $button The widget.
	 *
	 * @return string
	 */
	public function skin_print_template( $content, $button ) {

		if ( 'button' === $button->get_name() ) {
			return '';
		}

		return $content;
	}
}
