<?php
namespace HellixElementor\Modules\ImageBox\Skins;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use Elementor;
use Elementor\Skin_Base;
use Elementor\Control_Media;
use Elementor\Icons_Manager;
use Elementor\Utils;
use Elementor\Plugin;
use Elementor\Group_Control_Image_Size;
use Elementor\Controls_Manager;
use HellixElementor\Core\Utils as FE_Utils;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use Elementor\Group_Control_Typography;
use HellixElementor\Core\Utils as HXUtils;


/**
 * Skin for image Box Style 1.
 */
class Skin_Image_Box_Style_1 extends Skin_Base {

	/**
	 * Constructor.
	 *
	 * @param Elementor\Widget_Base $parent The widget settings.
	 * @return void
	 */
	public function __construct( Elementor\Widget_Base $parent ) {
		parent::__construct( $parent );
		add_filter( 'elementor/widget/print_template', [ $this, 'skin_print_template' ], 10, 2 );
		add_action( 'elementor/element/image-box/section_image/before_section_end', [ $this, 'section_image_controls' ], 10, 2 );
		add_action( 'elementor/element/image-box/section_style_image/before_section_end', [ $this, 'style_image_controls' ] );
		add_action( 'elementor/element/image-box/section_style_content/before_section_end', [ $this, 'style_content_controls' ] );
	}

	/**
	 * Get the name of the widget.
	 *
	 * @return string.
	 */
	public function get_id() {
		return 'image-box-v1';
	}

	/**
	 * Get the title of the skin.
	 *
	 * @return string
	 */
	public function get_title() {
		return esc_html__( 'Style 1', 'hellix-elementor' );
	}

	/**
	 * Style Image controls.
	 *
	 * @param array $widget The widget settings.
	 */
	public function style_image_controls( $widget ) {

		$this->parent = $widget;

		$disable_controls = [ 'section_style_image' ];

		foreach ( $disable_controls as $control ) {
			$widget->update_control(
				$control,
				[
					'condition' => [
						'_skin!' => 'image-box-v1',
					],
				]
			);
		}
	}

	/**
	 * Style Content controls.
	 *
	 * @param array $widget The widget settings.
	 */
	public function style_content_controls( $widget ) {
		$this->parent = $widget;

		$disable_controls = [ 'text_align', 'content_vertical_alignment' ];

		foreach ( $disable_controls as $control ) {
			$widget->update_control(
				$control,
				[
					'condition' => [
						'_skin!' => 'image-box-v1',
					],
				]
			);
		}

		$widget->update_control(
			'title_color',
			[
				'selectors' => [
					'{{WRAPPER}} .elementor-image-box-title' => 'color: {{VALUE}} !important;',
				],
			]
		);

		$widget->update_control(
			'description_color',
			[
				'selectors' => [
					'{{WRAPPER}} .elementor-image-box-description' => 'color: {{VALUE}} !important;',
				],
			]
		);

		$this->add_control(
			'btn_heading_description',
			[
				'label'     => esc_html__( 'Button', 'hellix-elementor' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'btn_description_color',
			[
				'label'     => esc_html__( 'Color', 'hellix-elementor' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .hellix-image-box-button' => 'color: {{VALUE}};',
				],
				'global'    => [
					'default' => Global_Colors::COLOR_TEXT,
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'btn_description_typography',
				'selector' => '{{WRAPPER}} .hellix-image-box-button',
				'global'   => [
					'default' => Global_Typography::TYPOGRAPHY_TEXT,
				],
			]
		);
	}

	/**
	 * Section Image controls.
	 *
	 * @param array $widget The widget settings.
	 */
	public function section_image_controls( $widget ) {

		$this->parent     = $widget;
		$disable_controls = [ 'thumbnail', 'position' ];

		foreach ( $disable_controls as $control ) {
			$widget->update_control(
				$control,
				[
					'condition' => [
						'_skin!' => 'image-box-v1',
					],
				]
			);
		}

		$widget->add_control(
			'button_text',
			[
				'label'     => esc_html__( 'Button Text', 'hellix-elementor' ),
				'type'      => Controls_Manager::TEXT,
				'default'   => esc_html__( 'READ MORE', 'hellix-elementor' ),
				'condition' => [
					'_skin' => 'image-box-v1',
				],
			]
		);
	}

	/**
	 * Skin Print Template.
	 *
	 * @param string                $content The skin output string.
	 * @param Elementor\Widget_Base $widget The widget.
	 *
	 * @return string
	 */
	public function skin_print_template( $content, $widget ) {
		if ( 'image-box' === $widget->get_name() ) {
			return '';
		}
		return $content;
	}

	/**
	 * Render the Image.
	 *
	 * @param array $settings The Widget settings.
	 */
	private function render_image( $settings ) {
		$widget = $this->parent;
		$link   = $settings['image']['url'];

		if ( empty( $settings['image']['url'] ) ) {
			return;
		}
		?>

		<div class="service__image">
			<?php echo wp_kses_post( HXUtils::render_bg_image( $link, '', '3:4' ) ); ?>
		</div>
		<?php

	}

	/**
	 * Render the title for the skin.
	 *
	 * @param array $settings The Widget settings.
	 */
	private function render_title( $settings ) {
		$widget     = $this->parent;
		$title      = $settings['title_text'];
		$has_title  = ! Utils::is_empty( $title );
		$tag        = $settings['title_size'];
		$title_html = $title;

		if ( ! $has_title ) {
			return;
		}

		$widget->add_render_attribute( 'title', 'class', [ 'elementor-image-box-title service__title text-white ' ] );
		?>
		<<?php echo esc_attr( $tag ); ?> <?php $widget->print_render_attribute_string( 'title' ); ?>>
			<?php echo wp_kses_post( $title_html ); ?>
		</<?php echo esc_attr( $tag ); ?>> 
		<?php
	}

	/**
	 * Render the Description.
	 *
	 * @param array $settings The Widget settings.
	 */
	private function render_description( $settings ) {
		$widget   = $this->parent;
		$desc     = $settings['description_text'];
		$has_desc = ! Utils::is_empty( $desc );

		if ( ! $has_desc ) {
			return;
		}

		$widget->add_render_attribute( 'description_text', 'class', 'elementor-image-box-description service__text text-light mt-3' );
		?>
		<p <?php $widget->print_render_attribute_string( 'description_text' ); ?>>
			<?php echo wp_kses_post( $settings['description_text'] ); ?>
		</p> 
		<?php
	}

	/**
	 * Render the content.
	 *
	 * @param array $settings The Widget settings.
	 */
	private function render_content( $settings ) {
		$widget   = $this->parent;
		$settings = $widget->get_settings_for_display();

		$has_content = ! Utils::is_empty( $settings['title_text'] ) || ! Utils::is_empty( $settings['description_text'] );

		if ( ! $has_content ) {
			return;
		}

		$widget->add_render_attribute( 'button', 'class', 'hellix-image-box-button button -simple text-beige-dark' );

		if ( ! empty( $settings['link']['url'] ) ) {
			$widget->add_link_attributes( 'button', $settings['link'] );
		}
		?>
		 <div class="service__content px-30 py-30 bg-dark">
			<?php
			$this->render_title( $settings );
			$this->render_description( $settings );
			?>
			<div class="service__button">
				<a <?php $widget->print_render_attribute_string( 'button' ); ?>> 
					<?php echo esc_html( $settings['button_text'] ); ?> 
				</a>
			</div>
		</div>
		<?php
	}

	/**
	 * Render the widget in the frontend.
	 */
	public function render() {
		$widget   = $this->parent;
		$settings = $widget->get_settings_for_display();
		?>
		<div data-anim="slide-up delay-1" class="service -image -hover">
		<?php
			$this->render_image( $settings );
			$this->render_content( $settings );
		?>
	   </div>
		<?php
	}
}
