<?php

namespace HellixElementor\Modules\Image;

use HellixElementor\Base\Module_Base;
use Elementor\Controls_Manager;
use Elementor\Core\Files\Assets\Files_Upload_Handler;
use Elementor\Core\Schemes;
use Elementor\Utils;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * The module class.
 */
class Module extends Module_Base {

	/**
	 * Instantiate the object.
	 */
	public function __construct() {
		parent::__construct();

		$this->add_actions();
	}

	/**
	 * Actions in this module.
	 */
	public function add_actions() {
		add_action( 'elementor/element/image/section_image/before_section_end', [ $this, 'add_css_classes_controls' ], 10 );
		add_filter( 'elementor/image_size/get_attachment_image_html', [ $this, 'image_html' ], 10, 4 );
		add_action( 'hellix-elementor/widget/image/render_content', [ $this, 'render_content' ], 10, 2 );
	}

	/**
	 * Get the name of the module.
	 *
	 * @return string
	 */
	public function get_name() {
		return 'override-image';
	}

	/**
	 * Add CSS Classes Controls to the widget.
	 *
	 * @param array $element The image widget.
	 */
	public function add_css_classes_controls( $element ) {

		$element->add_control(
			'image_class',
			[
				'label'       => esc_html__( 'Image Class', 'hellix-elementor' ),
				'type'        => Controls_Manager::TEXT,
				'title'       => esc_html__( 'Add your custom class for <img> tag  without the dot. e.g: my-class', 'hellix-elementor' ),
				'default'     => 'img-fluid',
				'label_block' => true,
				'description' => esc_html__( 'Additional CSS class that you want to apply to the img tag', 'hellix-elementor' ),
			]
		);

		$element->add_control(
			'wrapper',
			[
				'type'      => Controls_Manager::SWITCHER,
				'label'     => esc_html__( 'Need Wrapper Attributes ?', 'hellix-elementor' ),
				'default'   => 'no',
				'label_off' => esc_html__( 'Hide', 'hellix-elementor' ),
				'label_on'  => esc_html__( 'Show', 'hellix-elementor' ),
			]
		);
		$element->add_control(
			'wrapper_attr',
			[
				'label'       => __( 'Wrapper Attributes', 'hellix-elementor' ),
				'type'        => Controls_Manager::TEXTAREA,
				'dynamic'     => [
					'active' => true,
				],
				'placeholder' => esc_html__( 'key|value', 'hellix-elementor' ),
				/* translators: The delimiter character */
				'description' => sprintf( __( 'Set custom attributes for the wrapper element. Each attribute in a separate line. Separate attribute key from the value using %s character.', 'hellix-elementor' ), '<code>|</code>' ),
				'condition'   => [
					'wrapper' => 'yes',
				],
			]
		);
	}

	/**
	 * Before Render.
	 *
	 * @param string $html           The html.
	 * @param array  $settings       The settings.
	 * @param array  $image_size_key The image size key.
	 * @param array  $image_key      The image key.
	 * @return string
	 */
	public function image_html( $html, $settings, $image_size_key, $image_key ) {
		$enabled = Files_Upload_Handler::is_enabled();

		if ( $enabled && isset( $settings['inline_svg'] ) && 'yes' === $settings['inline_svg'] && isset( $settings['image']['url'] ) ) {

			if ( isset( $settings['image_class'] ) && ! empty( $settings['image_class'] ) ) {
				$html = '<div class="hellix-elementor-svg-wrapper ' . esc_attr( $settings['image_class'] ) . '">';
			} else {
				$html = '<div class="hellix-elementor-svg-wrapper">';
			}

			$html .= file_get_contents( $settings['image']['url'] );
			$html .= '</div>';

		} else {

			if ( isset( $settings['image_class'] ) && ! empty( $settings['image_class'] ) ) {

				if ( strpos( $html, 'class="' ) !== false ) {
					$html = str_replace( 'class="', 'class="' . esc_attr( $settings['image_class'] ) . ' ', $html );
				} else {
					$html = str_replace( '<img', '<img class="' . esc_attr( $settings['image_class'] ) . '"', $html );
				}
			}
		}

		return $html;
	}

	/**
	 * Wrap Start.
	 *
	 * @param  array $widget The widget settings.
	 */
	public function wrap_start( $widget ) {
		$settings    = $widget->get_settings_for_display();
		$has_wrapper = 'yes' === $settings['wrapper'] ? true : false;
		$wrapper_css = $settings['wrapper_attr'];
		$attributes  = explode( "\n", $wrapper_css );
		$wrapper     = '';
		$list_data   = '';
		if ( isset( $settings['wrapper'] ) && ( 'yes' === $settings['wrapper'] ) ) :
			$widget->add_render_attribute( 'img_wrapper', 'class', 'wrapper_css' );
			foreach ( $attributes as $attribute ) {
				if ( empty( trim( $attribute ) ) ) {
					continue;
				}
				$detail = explode( '|', $attribute );
				if ( ! empty( $detail[0] ) ) :
					$list_heading = $detail[0];
					if ( isset( $detail[1] ) ) {
						$list_data = $detail[1];
					}
				endif;
				$widget->add_render_attribute( 'img_wrapper', $list_heading, $list_data );
				$wrapper = '<div ' . $widget->get_render_attribute_string( 'img_wrapper' ) . '>';
			}
		endif;

		return $wrapper;
	}

	/**
	 * Wrap End.
	 *
	 * @param  array $widget The widget settings.
	 */
	public function wrap_end( $widget ) {
		$settings    = $widget->get_settings_for_display();
		$wrapper     = '';

		if ( isset( $settings['wrapper'] ) && ( 'yes' === $settings['wrapper'] ) ) :
			$wrapper = '</div>';
		endif;

		return $wrapper;
	}
	/**
	 * Render Content
	 *
	 * @param  string                 $content   The Content.
	 * @param \Elementor\Widget_Base $widget    The widget instance.
	 */
	public function render_content( $content, $widget ) {
		if ( 'image' === $widget->get_name() ) {
			$content = $this->wrap_start( $widget ) . $content . $this->wrap_end( $widget );
		}
		return $content;
	}
}
