<?php
namespace HellixElementor\Modules\PageSettings;

use Elementor\Controls_Manager;
use Elementor\Core\Base\Document;
use Elementor\Core\Base\Module as BaseModule;
use Elementor\Core\DocumentTypes\PageBase;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Elementor page templates module.
 *
 * Elementor page templates module handler class is responsible for registering
 * and managing Elementor page templates modules.
 *
 * @since 1.0.0
 */
class Module extends BaseModule {
	/**
	 * $post_id.
	 *
	 * @var integer
	 */
	protected $post_id = 0;
	/**
	 * $hx_page_options.
	 *
	 * @var array
	 */
	protected $hx_page_options = [];
	/**
	 * $static_contents.
	 *
	 * @var array
	 */
	protected $static_contents = [];

	/**
	 * Instantiate the object.
	 */
	public function __construct() {
		$this->hx_page_options = $this->hx_page_options();
		$this->static_contents = function_exists( 'hellix_static_content_options' ) ? hellix_static_content_options() : [];
		add_action( 'elementor/documents/register_controls', [ $this, 'action_register_template_control' ] );
		add_action( 'elementor/element/wp-post/section_page_style/before_section_end', [ $this, 'add_body_style_controls' ] );
		add_filter( 'update_post_metadata', [ $this, 'filter_update_meta' ], 10, 5 );
	}

	/**
	 * Get the name of the module.
	 *
	 * @return string
	 */
	public function get_name() {
		return 'override-page-settings';
	}

	/**
	 * Add Body Styles Controls.
	 *
	 * @param array $document The Document.
	 * @return void
	 */
	public function add_body_style_controls( $document ) {

		$document->add_control(
			'enable_overflow',
			[
				'label'     => esc_html__( 'Enable Overflow?', 'hellix-elementor' ),
				'type'      => Controls_Manager::SWITCHER,
				'selectors' => [
					'body' => 'overflow-x:visible !important;',
				],
			]
		);
	}

	/**
	 * Get Name.
	 *
	 * @return array
	 */
	public function get_special_settings_names() {
		return [
			// General.
			'general_body_classes',

			// Header.
			'header_hellix_enable_custom_header',
			'header_hellix_header_variant',
			'header_hellix_enable_topbar',
			'header_hellix_use_custom_logo',
			'header_hellix_custom_logo',
			'header_hellix_mobile_logo',
			'header_hellix_enable_navbar_search',
			'header_enable_cart',
			'header_enable_sticky',
			// Footer.
			'footer_enable_custom_footer',
			'footer_footer_skin',
			// 'footer_hellix_use_custom_logo',
			'footer_hellix_custom_elementor_logo',
			'footer_footer_copyright',
			'footer_footer_credit_text',
			'footer_footer_static_widgets',
			'footer_enable_static_content',

		];
	}

	/**
	 * Update Hellix Page Options.
	 *
	 * @param array $object_id Id.
	 * @param array $special_settings settings.
	 * @return void
	 */
	public function update_hx_page_option( $object_id, $special_settings ) {
		$_hx_page_options = $this->hx_page_options( $object_id );
		$hx_page_options  = ! empty( $_hx_page_options ) ? $_hx_page_options : [];

		$general_option_key     = 'general';
		$header_option_key      = 'header';
		$footer_option_key      = 'footer';
		$len_general_option_key = strlen( $general_option_key . '_' );
		$len_header_option_key  = strlen( $header_option_key . '_' );
		$len_footer_option_key  = strlen( $footer_option_key . '_' );

		foreach ( $special_settings as $key => $value ) {
			if ( substr( $key, 0, $len_general_option_key ) === $general_option_key . '_' ) {
				if ( ! isset( $hx_page_options[ $general_option_key ] ) ) {
					$hx_page_options[ $general_option_key ] = [];
				}
				$hx_page_options[ $general_option_key ][ substr( $key, $len_general_option_key ) ] = $value;
			} elseif ( substr( $key, 0, $len_header_option_key ) === $header_option_key . '_' ) {
				if ( ! isset( $hx_page_options[ $header_option_key ] ) ) {
					$hx_page_options[ $header_option_key ] = [];
				}
				$hx_page_options[ $header_option_key ][ substr( $key, $len_header_option_key ) ] = $value;
			} elseif ( substr( $key, 0, $len_footer_option_key ) === $footer_option_key . '_' ) {
				if ( ! isset( $hx_page_options[ $footer_option_key ] ) ) {
					$hx_page_options[ $footer_option_key ] = [];
				}
				$hx_page_options[ $footer_option_key ][ substr( $key, $len_footer_option_key ) ] = $value;
			} else {
				$hx_page_options[ $key ] = $value;
			}
		}

		if ( ! empty( $hx_page_options ) ) {
			$this->hx_page_options = $hx_page_options;
			update_metadata( 'post', $object_id, '_hx_page_options', $hx_page_options );
		}
	}

	/**
	 * Get Hellix Page Options.
	 *
	 * @param array  $option_name name.
	 * @param string $option_group group.
	 * @param string $dafault default.
	 */
	public function get_hx_page_options( $option_name, $option_group = '', $dafault = '' ) {
		$hx_page_options = $this->hx_page_options();

		if ( ! empty( $option_group ) && ! empty( $option_name ) ) {
			if ( isset( $hx_page_options[ $option_group ] ) && isset( $hx_page_options[ $option_group ][ $option_name ] ) ) {
				return $hx_page_options[ $option_group ][ $option_name ];
			}
		} elseif ( empty( $option_group ) && ! empty( $option_name ) ) {
			if ( isset( $hx_page_options[ $option_name ] ) ) {
				return $hx_page_options[ $option_name ];
			}
		}

		return $dafault;
	}

	/**
	 * Get Page Options.
	 *
	 * @param array $post_id post ID.
	 * @return array
	 */
	public function hx_page_options( $post_id = null ) {
		if ( ! empty( $this->hx_page_options ) ) {
			return $this->hx_page_options;
		}

		if ( ! $post_id ) {
			$post_id = $this->post_id;
		}

		$clean_meta_data = get_post_meta( $post_id, '_hx_page_options', true );
		$hx_page_options = maybe_unserialize( $clean_meta_data );

		if ( empty( $hx_page_options ) ) {
			$hx_page_options = [];
		} elseif ( ! empty( $hx_page_options ) && ! is_array( $hx_page_options ) ) {
			$hx_page_options = [];
		}

		$this->hx_page_options = $hx_page_options;
		return $hx_page_options;
	}

	/**
	 * Register template control.
	 *
	 * Adds custom controls to any given document.
	 *
	 * Fired by `update_post_metadata` action.
	 *
	 * @since 1.0.0
	 *
	 * @param Document $document The document instance.
	 */
	public function action_register_template_control( $document ) {
		$post_types = function_exists( 'hellix_option_enabled_post_types' ) ? hellix_option_enabled_post_types() : [ 'post', 'page' ];
		if ( $document instanceof PageBase && is_a( $document->get_main_post(), 'WP_Post' ) && in_array( $document->get_main_post()->post_type, $post_types ) ) {
			$this->post_id = $document->get_main_post()->ID;
			$this->register_template_control( $document );
		}
	}

	/**
	 * Register template control.
	 *
	 * @param Document $page   The document instance.
	 */
	public function register_template_control( $page ) {
		$this->add_general_controls( $page, 'general' );
		$this->add_header_controls( $page, 'header' );
		$this->add_footer_controls( $page, 'footer' );
	}

	/**
	 * Add General Controls.
	 *
	 * @param Document $page Page.
	 * @param string   $option_group group.
	 * @return void
	 */
	public function add_general_controls( Document $page, $option_group = '' ) {
		$page->start_injection(
			[
				'of'       => 'post_status',
				'fallback' => [
					'of' => 'post_title',
				],
			]
		);

		$page->add_control(
			'general_body_classes',
			[
				'label'   => esc_html__( 'Body Classes', 'hellix-elementor' ),
				'type'    => Controls_Manager::TEXT,
				'default' => $this->get_hx_page_options( 'body_classes', $option_group ),
			]
		);

		$page->end_injection();
	}

	/**
	 * Add Header Controls.
	 *
	 * @param Document $page Page.
	 * @param string   $option_group group.
	 * @return void
	 */
	public function add_header_controls( Document $page, $option_group = '' ) {
		$page->start_controls_section(
			'document_settings_header',
			[
				'label' => esc_html__( 'Headers', 'hellix-elementor' ),
				'tab'   => Controls_Manager::TAB_SETTINGS,
			]
		);

		$page->add_control(
			'header_hellix_enable_custom_header',
			[
				'label'     => esc_html__( 'Custom Header', 'hellix-elementor' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => $this->get_hx_page_options( 'hellix_enable_custom_header', 'header' ),
				'label_on'  => esc_html__( 'Enable', 'hellix-elementor' ),
				'label_off' => esc_html__( 'Disable', 'hellix-elementor' ),
			]
		);

		$page->add_control(
			'header_hellix_header_variant',
			[
				'label'     => esc_html__( 'Navbar Variant', 'hellix-elementor' ),
				'type'      => Controls_Manager::SELECT,
				'options'   => [
					'v1'  => esc_html__( 'Header v1', 'hellix-elementor' ),
					'v2'  => esc_html__( 'Header v2', 'hellix-elementor' ),
					'v3'  => esc_html__( 'Header v3', 'hellix-elementor' ),
					'v4'  => esc_html__( 'Header v4', 'hellix-elementor' ),
					'v5'  => esc_html__( 'Header v5', 'hellix-elementor' ),
					'v6'  => esc_html__( 'Header v6', 'hellix-elementor' ),
					'v7'  => esc_html__( 'Header v7', 'hellix-elementor' ),
					'v8'  => esc_html__( 'Header v8', 'hellix-elementor' ),
					'v9'  => esc_html__( 'Header v9', 'hellix-elementor' ),
					'v10' => esc_html__( 'Header v10', 'hellix-elementor' ),
				],
				'condition' => [
					'header_hellix_enable_custom_header' => 'yes',
				],
				'default'   => $this->get_hx_page_options( 'hellix_header_variant', $option_group, 'v1' ),
			]
		);

		$page->add_control(
			'header_hellix_use_custom_logo',
			[
				'label'     => esc_html__( 'Use Custom Logo?', 'hellix-elementor' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => $this->get_hx_page_options( 'hellix_use_custom_logo', $option_group ),
				'label_on'  => esc_html__( 'Enable', 'hellix-elementor' ),
				'label_off' => esc_html__( 'Disable', 'hellix-elementor' ),
				'condition' => [
					'header_hellix_enable_custom_header' => 'yes',
				],
			]
		);

		$page->add_control(
			'header_hellix_custom_logo',
			[
				'label'     => esc_html__( 'Upload Logo', 'hellix-elementor' ),
				'type'      => Controls_Manager::MEDIA,
				'default'   => [
					'id'  => $this->get_hx_page_options( 'hellix_custom_logo', $option_group ),
					'url' => (string) wp_get_attachment_image_url( $this->get_hx_page_options( 'hellix_custom_logo', $option_group ) ),
				],

				'condition' => [
					'header_hellix_enable_custom_header' => 'yes',
					'header_hellix_use_custom_logo'      => 'yes',
				],
			]
		);

		$page->add_control(
			'header_hellix_mobile_logo',
			[
				'label'     => esc_html__( 'Upload Moblie Logo', 'hellix-elementor' ),
				'type'      => Controls_Manager::MEDIA,
				'default'   => [
					'id'  => $this->get_hx_page_options( 'hellix_custom_logo', $option_group ),
					'url' => (string) wp_get_attachment_image_url( $this->get_hx_page_options( 'hellix_mobile_logo', $option_group ) ),
				],

				'condition' => [
					'header_hellix_enable_custom_header' => 'yes',
					'header_hellix_use_custom_logo'      => 'yes',
					'header_hellix_header_variant'       => 'v10',
				],
			]
		);

		$page->add_control(
			'header_hellix_enable_topbar',
			[
				'label'     => esc_html__( 'Enable Topbar', 'hellix-elementor' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => $this->get_hx_page_options( 'hellix_enable_topbar', $option_group ),
				'label_on'  => esc_html__( 'Enable', 'hellix-elementor' ),
				'label_off' => esc_html__( 'Disable', 'hellix-elementor' ),
				'condition' => [
					'header_hellix_enable_custom_header' => 'yes',
					'header_hellix_header_variant'       => [ 'v2', 'v3' ],
				],
			]
		);

		$page->add_control(
			'header_hellix_enable_navbar_search',
			[
				'label'     => esc_html__( 'Enable Search', 'hellix-elementor' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => $this->get_hx_page_options( 'hellix_enable_navbar_search', $option_group ),
				'label_on'  => esc_html__( 'Enable', 'hellix-elementor' ),
				'label_off' => esc_html__( 'Disable', 'hellix-elementor' ),
				'condition' => [
					'header_hellix_enable_custom_header' => 'yes',
					'header_hellix_header_variant!'      => 'v10',
				],
			]
		);

		if ( function_exists( 'hellix_is_woocommerce_activated' ) && hellix_is_woocommerce_activated() ) {

			$page->add_control(
				'header_enable_cart',
				[
					'label'     => esc_html__( 'Enable Cart', 'hellix-elementor' ),
					'type'      => Controls_Manager::SWITCHER,
					'default'   => $this->get_hx_page_options( 'enable_cart', $option_group ),
					'label_on'  => esc_html__( 'Enable', 'hellix-elementor' ),
					'label_off' => esc_html__( 'Disable', 'hellix-elementor' ),
					'condition' => [
						'header_hellix_enable_custom_header' => 'yes',
						'header_hellix_header_variant!' => 'v10',
					],
				]
			);
		}

		$page->add_control(
			'header_enable_sticky',
			[
				'label'     => esc_html__( 'Header Sticky?', 'hellix-elementor' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => $this->get_hx_page_options( 'enable_sticky', $option_group ),
				'label_on'  => esc_html__( 'Enable', 'hellix-elementor' ),
				'label_off' => esc_html__( 'Disable', 'hellix-elementor' ),
				'condition' => [
					'header_hellix_enable_custom_header' => 'yes',
					'header_hellix_header_variant!'      => 'v10',
				],
			]
		);

		$page->end_controls_section();
	}

	/**
	 * Add Footer Controls.
	 *
	 * @param Document $page Page.
	 * @param string   $option_group group.
	 * @return void
	 */
	public function add_footer_controls( Document $page, $option_group = '' ) {
		$page->start_controls_section(
			'document_settings_footer',
			[
				'label' => esc_html__( 'Footer', 'hellix-elementor' ),
				'tab'   => Controls_Manager::TAB_SETTINGS,
			]
		);

		$page->add_control(
			'footer_enable_custom_footer',
			[
				'label'     => esc_html__( 'Custom Footer', 'hellix-elementor' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => $this->get_hx_page_options( 'enable_custom_footer', $option_group, 'no' ),
				'label_on'  => esc_html__( 'Enable', 'hellix-elementor' ),
				'label_off' => esc_html__( 'Disable', 'hellix-elementor' ),
			]
		);

		$page->add_control(
			'footer_enable_static_content',
			[
				'label'     => esc_html__( 'Footer Type', 'hellix-elementor' ),
				'type'      => Controls_Manager::SELECT,
				'options'   => [
					'no'  => esc_html__( 'Default', 'hellix-elementor' ),
					'yes' => esc_html__( 'Static Footer', 'hellix-elementor' ),
				],
				'condition' => [
					'footer_enable_custom_footer' => 'yes',
				],
				'default'   => $this->get_hx_page_options( 'enable_static_content', $option_group, 'no' ),
			]
		);

		if ( function_exists( 'hellix_is_mas_static_content_activated' ) && hellix_is_mas_static_content_activated() ) {
			$page->add_control(
				'footer_footer_static_widgets',
				[
					'label'     => esc_html__( 'Footer Static Widgets', 'hellix-elementor' ),
					'type'      => Controls_Manager::SELECT,
					'options'   => $this->static_contents,
					'condition' => [
						'footer_enable_static_content' => 'yes',
						'footer_enable_custom_footer'  => 'yes',
					],
					'default'   => $this->get_hx_page_options( 'footer_static_widgets', $option_group, '' ),
				]
			);
		}

		$page->add_control(
			'footer_footer_skin',
			[
				'label'     => esc_html__( 'Footer Varaiant', 'hellix-elementor' ),
				'type'      => Controls_Manager::SELECT,
				'options'   => [
					'white' => esc_html__( 'White', 'hellix-elementor' ),
					'dark'  => esc_html__( 'Dark', 'hellix-elementor' ),
				],
				'condition' => [
					'footer_enable_custom_footer' => 'yes',
				],
				'default'   => $this->get_hx_page_options( 'footer_skin', $option_group, 'dark' ),
			]
		);

		$page->add_control(
			'footer_hellix_custom_elementor_logo',
			[
				'label'     => esc_html__( 'Upload Logo', 'hellix-elementor' ),
				'type'      => Controls_Manager::MEDIA,
				'default'   => [
					'id'  => $this->get_hx_page_options( 'hellix_custom_elementor_logo', $option_group ),
					'url' => (string) wp_get_attachment_image_url( $this->get_hx_page_options( 'hellix_custom_elementor_logo', $option_group ) ),
				],

				'condition' => [
					'footer_enable_custom_footer' => 'yes',
				],
			]
		);

		$page->add_control(
			'footer_footer_copyright',
			[
				'label'     => esc_html__( 'Copyright Text', 'hellix-elementor' ),
				'type'      => Controls_Manager::TEXTAREA,
				'default'   => esc_html__( '© 2022 Hellix. All rights reserved.', 'hellix-elementor' ),
				'condition' => [
					'footer_enable_custom_footer' => 'yes',
				],
			]
		);

		$page->add_control(
			'footer_footer_credit_text',
			[
				'label'     => esc_html__( 'Credit Text', 'hellix-elementor' ),
				'type'      => Controls_Manager::TEXTAREA,
				'default'   => esc_html__( 'Built by MadrasThemes', 'hellix-elementor' ),
				'condition' => [
					'footer_enable_custom_footer' => 'yes',
				],
			]
		);

		$page->end_controls_section();

	}

	/**
	 * Filter metadata update.
	 *
	 * Filters whether to update metadata of a specific type.
	 *
	 * Elementor don't allow WordPress to update the parent page template
	 * during `wp_update_post`.
	 *
	 * Fired by `update_{$meta_type}_metadata` filter.
	 *
	 * @since 1.0.0
	 *
	 * @param bool   $check     Whether to allow updating metadata for the given type.
	 * @param int    $object_id Object ID.
	 * @param string $meta_key  Meta key.
	 * @param string $meta_value  Meta Value.
	 * @param string $prev_value  previous value.
	 *
	 * @return bool Whether to allow updating metadata of a specific type.
	 */
	public function filter_update_meta( $check, $object_id, $meta_key, $meta_value, $prev_value ) {
		if ( '_elementor_page_settings' === $meta_key ) {
			$current_check = $check;
			if ( ! empty( $meta_value ) && is_array( $meta_value ) ) {
				$special_settings_names = $this->get_special_settings_names();
				$special_settings       = [];
				foreach ( $special_settings_names as $name ) {
					if ( isset( $meta_value[ $name ] ) ) {
						$special_settings[ $name ] = $meta_value[ $name ];
						unset( $meta_value[ $name ] );
						$current_check = false;
					}
				}
				if ( false === $current_check ) {
					update_metadata( 'post', $object_id, $meta_key, $meta_value, $prev_value );
					$this->update_hx_page_option( $object_id, $special_settings );
					return $current_check;
				}
			}
		}

		return $check;
	}
}
