<?php

namespace HellixElementor\Modules\Tabs\Skins;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use Elementor;
use Elementor\Icons_Manager;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Skin_Base;
use Elementor\Group_Control_Typography;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use HellixElementor\Plugin;
use Elementor\Repeater;
use Elementor\Plugin as ElementorPlugin;
use Elementor\Utils;

/**
 * Skin for Tab Services.
 */
class Skin_Tabs_Services extends Skin_Base {
	/**
	 * Get the id of the skin.
	 *
	 * @return string
	 */
	public function get_id() {
		return 'tabs-services';
	}
	/**
	 * Get the title of the skin.
	 *
	 * @return string
	 */
	public function get_title() {
		return esc_html__( 'Services', 'hellix-elementor' );
	}
	/**
	 * Register Control Actions in the skin..
	 *
	 * @return void
	 */
	protected function _register_controls_actions() {
		add_filter( 'hellix-elementor/widget/tabs/print_template', [ $this, 'skin_print_template' ], 10, 2 );
		add_action( 'elementor/element/tabs/section_tabs/before_section_end', [ $this, 'additional_content_controls' ], 20 );
		add_action( 'elementor/element/tabs/section_tabs_style/after_section_end', [ $this, 'additional_style_controls' ], 20 );
	}

	/**
	 * Adding additional content controls to content tab.
	 *
	 * @param array $widget The widget settings.
	 * @return void
	 */
	public function additional_content_controls( $widget ) {

		$update_control_ids = [
			'tabs',
			'tabs_align_horizontal',
			'tabs_align_vertical',
			'type',
		];

		foreach ( $update_control_ids as $update_control_id ) {
			$widget->update_control(
				$update_control_id,
				[
					'condition' => [
						'_skin' => '',
					],
				]
			);
		}

		$repeater = new Repeater();

		$default_icon = [
			'value' => 'icon-pencil',
		];

		$repeater->add_control(
			'tab_name',
			[
				'label' => esc_html__( 'Title', 'hellix-elementor' ),
				'type'  => Controls_Manager::TEXT,
			]
		);

		$repeater->add_control(
			'content_id',
			[
				'label'       => esc_html__( 'Tab ID', 'hellix-elementor' ),
				'type'        => Controls_Manager::TEXT,
				'placeholder' => esc_html__( 'Tab ID', 'hellix-elementor' ),
				'description' => esc_html__( 'Put the same id in content class', 'hellix-elementor' ),
			]
		);

		$repeater->add_control(
			'icon_item',
			[
				'label'            => esc_html__( 'Icon', 'hellix-elementor' ),
				'type'             => Controls_Manager::ICONS,
				'fa4compatibility' => 'item_icon',
				'default'          => $default_icon,
			]
		);

		$repeater->add_control(
			'icon_css',
			[
				'label' => esc_html__( 'Icon CSS', 'hellix-elementor' ),
				'type'  => Controls_Manager::TEXT,
			]
		);

		$this->add_control(
			'tabs_title',
			[
				'label'       => esc_html__( 'Tabs', 'hellix-elementor' ),
				'type'        => Controls_Manager::REPEATER,
				'fields'      => $repeater->get_controls(),
				'default'     => [
					[
						'tab_name' => esc_html__( 'Tab #1', 'hellix-elementor' ),
					],
					[
						'tab_name' => esc_html__( 'Tab #2', 'hellix-elementor' ),
					],
					[
						'tab_name' => esc_html__( 'Tab #3', 'hellix-elementor' ),
					],
				],
				'title_field' => '{{{ tab_name }}}',
			],
			[
				'position' => [
					'at' => 'after',
					'of' => '_skin',
				],
			]
		);
	}
	/**
	 * Adding additional style controls to style tab.
	 *
	 * @param array $widget The widget settings.
	 * @return void
	 */
	public function additional_style_controls( $widget ) {

		$this->parent = $widget;

		$update_control_ids = [ 'section_tabs_style' ];

		foreach ( $update_control_ids as $update_control_id ) {
			$widget->update_control(
				$update_control_id,
				[
					'condition' => [
						'_skin' => '',
					],
				]
			);
		}

		$this->start_controls_section(
			'section_tabs_style',
			[
				'label' => esc_html__( 'Tabs', 'hellix-elementor' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'heading_title',
			[
				'label'     => esc_html__( 'Tab Title', 'hellix-elementor' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'color',
			[
				'label'     => esc_html__( 'Title Color', 'hellix-elementor' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .hx-tabs__title' => 'color: {{VALUE}} !important;',
				],

			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'label'    => esc_html__( 'Title typography', 'hellix-elementor' ),
				'name'     => 'title_typography',
				'selector' => '{{WRAPPER}} .hx-tabs__title',
				'global'   => [
					'default' => Global_Typography::TYPOGRAPHY_PRIMARY,
				],
			]
		);

		$this->add_control(
			'icon_color',
			[
				'label'     => esc_html__( 'Icon Color', 'hellix-elementor' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .hx_icon-color' => 'color: {{VALUE}} !important;',
				],
			]
		);

		$this->end_controls_section();
	}
	/**
	 * Render the skin in the frontend.
	 */
	public function render() {
		$parent   = $this->parent;
		$settings = $parent->get_settings_for_display();

		$skin_control_ids = [
			'tabs_title',
			'icon_item',
		];

		$skin_settings = [];

		foreach ( $skin_control_ids as $skin_control_id ) {
			$skin_settings[ $skin_control_id ] = $this->get_instance_value( $skin_control_id );
		}

		?>
		<div class="tabs__controls js-tabs-controls">
			<?php foreach ( $skin_settings['tabs_title'] as $index => $item ) : ?>
				<?php
					$tab_count      = $index + 1;
					$tab_content_id = $item['content_id'];
					$parent->add_render_attribute(
						'tab-title-' . $tab_count,
						[
							'class'           => 'tabs__button col-12' . ( 1 === $tab_count ? ' is-active' : '' ) . ( 1 !== $tab_count ? ' mt-20' : '' ),
							'data-tab-target' => '.' . $tab_content_id,
							'type'            => 'button',
							'id'              => $item['_id'] . $tab_count,
						]
					);
				if ( ! isset( $item['icon_item']['value']['url'] ) ) {
					$icon_css = ' hx_icon-color text-xl text-accent mr-16';
					if ( $item['icon_css'] ) {
						$icon_css .= ' ' . $item['icon_css'];
					}
					$parent->add_render_attribute( 'hx-item-icon-' . $item['_id'], 'class', $item['icon_item']['value'] . $icon_css );
				}
				if ( isset( $item['icon_item']['value']['url'] ) ) {
					?>
						<?php Icons_Manager::render_icon( $item['icon_item'], [ 'aria-hidden' => 'true' ] ); ?>
						<?php
				}
				?>
				<button <?php $parent->print_render_attribute_string( 'tab-title-' . $tab_count ); ?>>
					<i <?php $parent->print_render_attribute_string( 'hx-item-icon-' . $item['_id'] ); ?>></i>
					<span class="hx-tabs__title"><?php echo esc_html( $item['tab_name'] ); ?></span>
				</button>
			<?php endforeach; ?>
		</div>
		<?php
		$this->render_script();

	}
	/**
	 * Skin print template.
	 *
	 * @param array $content the content.
	 * @param array $tabs widget name.
	 * @return string
	 */
	public function skin_print_template( $content, $tabs ) {

		if ( 'tabs' === $tabs->get_name() ) {
			return '';
		}

		return $content;
	}


	/**
	 * Render script in the editor.
	 */
	public function render_script() {
		if ( ElementorPlugin::$instance->editor->is_edit_mode() ) :
			?>
			<script type="text/javascript">
				const Tabs = (function() {
				function init() {
					const targets = document.querySelectorAll(".js-tabs");
					if (!targets) return;

					targets.forEach(el => {
					singleTab(el)
					})
				}

				function singleTab(target) {
					const controls = target.querySelector('.js-tabs-controls');
					const controlsItems = target.querySelectorAll('.js-tabs-controls .tabs__button');
					const content = target.querySelector('.js-tabs-content');

					for (let l = 0; l < controlsItems.length; l++) {
						const el = controlsItems[l];	
						el.addEventListener("click", (e) => {
							const selector = el.getAttribute('data-tab-target');

							controls.querySelector('.is-active').classList.remove('is-active')
							content.querySelector('.is-active').classList.remove('is-active')

							el.classList.add('is-active')
							content.querySelector(selector).classList.add('is-active')
						});
					}
				}

				return {
					init: init,
				}
				})();
				Tabs.init();
			</script>
			<?php
		endif;
	}
}
